/*
 * Copyright 2018 ConsenSys AG.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package tech.pegasys.pantheon.tests.acceptance.dsl.condition.clique;

import static org.assertj.core.api.Java6Assertions.assertThat;
import static tech.pegasys.pantheon.tests.acceptance.dsl.WaitUtils.waitFor;
import static tech.pegasys.pantheon.tests.acceptance.dsl.condition.clique.ExpectNonceVote.CLIQUE_NONCE_VOTE.AUTH;

import tech.pegasys.pantheon.tests.acceptance.dsl.condition.Condition;
import tech.pegasys.pantheon.tests.acceptance.dsl.node.Node;
import tech.pegasys.pantheon.tests.acceptance.dsl.transaction.eth.EthTransactions;

public class ExpectNonceVote implements Condition {
  private static final String NONCE_AUTH = "0xffffffffffffffff";
  private static final String NONCE_DROP = "0x0000000000000000";
  private final EthTransactions eth;
  private final String expectedNonce;

  public enum CLIQUE_NONCE_VOTE {
    AUTH,
    DROP
  }

  public ExpectNonceVote(final EthTransactions eth, final CLIQUE_NONCE_VOTE vote) {
    this.eth = eth;
    this.expectedNonce = vote == AUTH ? NONCE_AUTH : NONCE_DROP;
  }

  @Override
  public void verify(final Node node) {
    waitFor(() -> assertThat(node.execute(eth.block()).getNonceRaw()).isEqualTo(expectedNonce));
  }
}
